
"""VirtualHostTool
"""

# requires the VirtualHostFolder product to be installed in the Zope root

from Products.CMFCore.utils import UniqueObject, getToolByName
from OFS.SimpleItem import SimpleItem
from Acquisition import aq_inner, aq_parent

from Globals import InitializeClass, DTMLFile, PersistentMapping
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.CMFCore import CMFCorePermissions
from utils import _dtmldir
from urllib import quote

from Products.VirtualHostFolder import IHost


class VirtualHostTool (UniqueObject, SimpleItem):
    id = 'portal_virtualhost'
    meta_type = 'CMF Virtual Host Tool'

    def getId(self): return self.id

    def __init__(self):
        self.vhfPath="/VHF" # holds path reference to virtual host folder (tool tries to find the VHF if that is not the id)

    security = ClassSecurityInfo()
    security.declareObjectPublic()

    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' } ,
                       { 'label' : 'Skins', 'action' : 'manage_skins' }
                     ) + SimpleItem.manage_options


    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = DTMLFile( 'explainVirtualHostTool', _dtmldir )

    security.declarePrivate( 'manage_skins_form' ) # because you should only access it through manage_skins
    manage_skins_form = DTMLFile( 'skinSetting', _dtmldir )

    security.declarePublic( 'manage_skins' ) # because the security assertions are not standard, and are within the method itself
    def manage_skins(self, manager_skins=[], owner_skins=[], default_skin=None, request=None):
        """provides both a general way for adding skins and a through-the-web
        interface"""
        if not request and hasattr(self, 'REQUEST'): request=self.REQUEST
        """sets the skins for the current virtual host, with security settings
        based on the current static virtual root"""
        if request is not None and not ( manager_skins or owner_skins or default_skin):
            return self.manage_skins_form(self, request)
        msg=[]
        mfunc=msg.append
        lvl=self.getSecurityLevel()
        if manager_skins:
            if lvl>=3: self.setSkinManagerChoices(manager_skins, request, mfunc)
            elif mfunc: mfunc("You do not have the necessary permission to set the skin manager choices.")
        if owner_skins:
            if lvl>=2: self.setSkinOwnerChoices(owner_skins, request, mfunc)
            elif mfunc: mfunc("You do not have the necessary permission to set the skin owner choices.")
            ##raise "poop: "+repr(lvl)+" "+repr(manager_skins)+" "+repr(owner_skins)+" "+repr(default_skin)+" "+repr(msg)
        if default_skin:
            if lvl>=2: self.setDefaultSkinName(default_skin, request, mfunc)
            elif mfunc: mfunc("You do not have the necessary permission to set the default skin name.")
        if request is not None:
            return self.manage_skins_form(self, request, messages=msg)
        return 1


    #
    #   'portal_virtualhost' interface methods
    #

    security.declarePublic( 'getTitle' )
    def getTitle( self, request=None ):
        """ get the host or portal title """
        return self.getProperty( 'title', request ) or getToolByName( self, 'portal_properties').title()

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getRootURL' )
    def getRootURL( self, request=None ):
        """
        Return the host's canonical root URL in regard to entire Zope
        """
        try:
            return self.getVirtualHostFolder().getCanonicalVirtualRootURL(request)
        except:
            return '/'

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getActualRootURL' )
    def getActualRootURL( self, request=None ):
        """
        Return the host's actual current root URL in regard to entire Zope
        """
        try:
            return self.getVirtualHostFolder().getActualVirtualRootURL(request)
        except:
            return '/'

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getRootPath' )
    def getRootPath( self, request=None ):
        """
        Return the host's canonical root path in regard to entire Zope
        """
        try:
            return self.getRootURL( request ).split("/")[1:]
        except:
            return []

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getActualRootPath' )
    def getActualRootPath( self, request=None ):
        """
        Return the host's actual root path in regard to entire Zope
        """
        try:
            return self.getActualRootURL( request ).split("/")[1:]
        except:
            return []

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getRootObject' )
    def getRootObject( self, request = None ):
        """ Return the canonical root object """
        try:
            return self.getVirtualHostFolder().getCanonicalVirtualRootObject(request)
        except:
            return self.getPhysicalRoot()

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getActualRootObject' )
    def getActualRootObject( self, request = None ):
        """ Return the actual root object """
        try:
            return self.getVirtualHostFolder().getActualVirtualRootObject(request)
        except:
            return self.getPhysicalRoot()

    security.declarePublic( 'getProperty' )
    def getProperty( self, attr, request=None ):
        """
        Returns the stored value of the given name, if any, for the current VH (or none).
        """
        vh= self.getVirtualHost( request )
        return vh and vh.getProperty( attr ) or None


    security.declarePublic( 'getProperties' )
    def getProperties( self, request=None ):
        """
        Returns a list of property tuples (name, value) for the current VH.
        """
        vh = self.getVirtualHost( request )
        return vh and vh.propertyItems( attr ) or None
        ## return vh and [x for x in vh.propertyItems() if x[0] not in self.secure_names] or []

    security.declarePrivate( 'getVirtualHostFolder' )
    def getVirtualHostFolder( self ):
        """returns the VHF object, if it can"""
        result=None
        try:
            if not self.vhfPath: raise # so we can go to the except
            result=self.unrestrictedTraverse( self.vhfPath ) # will automatically raise an error and go to except if it fails
            if result.meta_type != "Virtual Host Folder": raise # so we can go to the except
        except:
            result=self.getPhysicalRoot().objectItems( "Virtual Host Folder" )
            if not result or not len(result):
                raise LookupError ("An object with 'Virtual Host Folder' meta_type is not found in Zope root.  The Zope instance must contain a VHF in the root to use the CMF Virtual Host Tool.")
            self.vhfPath="/"+result[0][0]
            result=result[0][1]
        return result

    security.declarePrivate( 'getVirtualHost' )
    def getVirtualHost( self, request=None ):
        """returns the current virtual host object, if it can"""
        if not request and hasattr(self, "REQUEST"): request=self.REQUEST
        try:
            return self.getVirtualHostFolder().getCurrentVirtualHostObject( request )
        except:
            return None

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getSkinManagerChoices' )
    def getSkinManagerChoices( self, request=None ):
        """gets the skin names approved by the manager for the current host"""
        host=self.getVirtualHost(request)
        if host and IHost.ICMFSkinGetterHost.isImplementedBy(host):
            results= host.getCMFManagerSkins()
            if results: return results
        return getToolByName( self, "portal_skins" ).getSkinSelections()

    security.declarePrivate( "setSkinManagerChoices")
    def setSkinManagerChoices( self, skinNames, request=None, mfunc=None ):
        """sets the manager skin names.  Private because through the web setting
        of this is tricky: we should be checking the permissions on the static
        virtual root, not anywhere else, for all of the skin setters."""
        host=self.getVirtualHost(request)
        if host and IHost.ICMFSkinSetterHost.isImplementedBy(host):
            return host.setCMFManagerSkins(skinNames, mfunc)
        if mfunc: mfunc("This host does not support setting CMF skins")
        return 0


    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getSkinOwnerChoices' )
    def getSkinOwnerChoices( self, request=None ):
        """gets the skin names approved by the owner for the current host (as
        already limited by the manager)"""
        host=self.getVirtualHost(request)
        if host and IHost.ICMFSkinGetterHost.isImplementedBy(host):
            results= host.getCMFOwnerSkins()
            if results: return results
        return getToolByName( self, "portal_skins" ).getSkinSelections()

    security.declarePrivate( "setSkinOwnerChoices")
    def setSkinOwnerChoices( self, skinNames, request=None, mfunc=None ):
        """sets the owner skin names.  Private because through the web setting
        of this is tricky: we should be checking the permissions on the static
        virtual root, not anywhere else, for all of the skin setters."""
        host=self.getVirtualHost(request)
        if host and IHost.ICMFSkinSetterHost.isImplementedBy(host):
            return host.setCMFOwnerSkins(skinNames, mfunc)
        if mfunc: mfunc("This host does not support setting CMF skins")
        return 0

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getDefaultSkinName' )
    def getDefaultSkinName( self, request=None ):
        """gets the default skin for the current host"""
        host=self.getVirtualHost(request)
        if host and IHost.ICMFSkinGetterHost.isImplementedBy(host):
            result= host.getCMFDefaultSkin()
            if result: return result
        return getToolByName( self, "portal_skins" ).getDefaultSkin()

    security.declarePrivate( "setDefaultSkinName")
    def setDefaultSkinName( self, skinName, request=None, mfunc=None ):
        """sets the default skin.  Private because through the web setting
        of this is tricky: we should be checking the permissions on the static
        virtual root, not anywhere else, for all of the skin setters."""
        host=self.getVirtualHost(request)
        if host and IHost.ICMFSkinSetterHost.isImplementedBy(host):
            return host.setCMFDefaultSkin(skinName, mfunc)
        if mfunc: mfunc("This host does not support setting CMF skins")
        return 0

    security.declarePublic( 'getSecurityLevel' )
    def getSecurityLevel( self, request=None ):
        """gets the security level of the current user for the static virtual
        root of the current host."""
        obj=self.getVirtualHost(request)
        if not obj: return 0
        obj=obj.getVirtualRootObject() # this is the static root
        if not obj: return 0
        securityManager=getSecurityManager()
        ck=securityManager.checkPermission
        return ck(
            CMFCorePermissions.ManagePortal, obj) and 3 or ck(
            CMFCorePermissions.ReviewPortalContent, obj) and 2 or ck(
            CMFCorePermissions.AddPortalContent, obj) and 1 # or 0

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'setProperty' )
    def setProperty( self, attr, val, request=None ):
        """
        For host as indicated by hostkey, sets attr to val
        """
        raise NotImplementedError
        ### yup; we need some more thought on this
        ## self.getVirtualHost( request ).manage_changeProperties({attr:val})
        return 1


    ##security.declarePublic( 'getBreadcrumbs' )
    ##def getBreadcrumbs(self, root_name="home", includeQueryString=0, request=None):
    ##    """ list of "id", "url" dicts, derived simply from actual URL passed to
    ##    Zope, without any title"""
    ##    if not request and hasattr(self, "REQUEST"): request=self.REQUEST
    ##    try:
    ##        return self.getVirtualHostFolder().getCleanOriginalRequestList(request, rootName)
    ##    except:
    ##        return []

    security.declarePublic( 'getOriginalRequestURL' )
    def getOriginalRequestURL(self, request=None):
        """ the path part of the URL as originally entered """
        try:
            return self.getVirtualHostFolder().getOriginalRequestURL(request)
        except:
            if not request and hasattr(self, "REQUEST"): request=self.REQUEST
            return "/"+"/".join(request.steps) # I think

    security.declarePublic( 'getPathToContent' )
    def getPathToContent(self, obj, request=None):
        """ get the path to an object, taking into account aliases and the
        like """
        try:
            return self.getVirtualHost(request).getVirtualPathToObject( obj, request )
        except:
            if not request and hasattr(self, "REQUEST"): request=self.REQUEST
            rpp = request.get('VirtualRootPhysicalPath', ('',))
            spp = self.getPhysicalPath()
            i = 0
            for name in rpp[:len(spp)]:
                if spp[i] == name:
                    i = i + 1
                else:
                    break
            return map(quote, spp[i:])

    security.declarePublic( 'getURLToContent' )
    def getURLToContent(self, obj, request=None):
        """ get the URL to an object, taking into account aliases and the
        like """
        try:
            return self.getVirtualHost(request).getVirtualURLToObject( obj, request )
        except:
            return  obj.absolute_url()

    security.declarePublic( 'getFullBreadcrumbs' )
    def getBreadcrumbs(self, obj, include_root=1, root_name='home', ignoreIds=("index_html","talkback"), careful=0, request=None):
        """returns id title and url of each object in acquisition path of obj"""
        host=self.getVirtualHost(request)
        res = []
        if host:
            if careful:
                objURL=obj and host.getVirtualURLToObject(obj, request) or ""
                while obj and objURL != "/":
                    if objURL:
                        objId=obj.getId()
                        if objId and objId not in ignoreIds:
                            res.insert(0, {'id' : objId, 'title' : obj.title_or_id(), 'url' : objURL } )
                    obj=obj.aq_parent
                    objURL=obj and host.getVirtualURLToObject(obj, request) or ""
            else:
                objPath=obj and host.getVirtualPathToObject(obj, request) or []
                for i in range( len(objPath), 0, -1):
                    now=objPath[:i]
                    objId=obj.getId()
                    if objId and objId==now[-1] and objId not in ignoreIds:
                        res.insert(0, {'id' : objId, 'title' : obj.title_or_id(), 'url' : '/'+'/'.join(now) } )
                    obj=obj.aq_parent
            if include_root:
                res.insert(0,{'id' : root_name, 'title' : self.getTitle( request ), 'url' : '/' } )
        else:
            portal_url = obj.portal_url()
            if include_root:
                res.insert(0,{'id' : root_name, 'title' : self.getTitle( request ), 'url' : '/' } )
            relative = obj.portal_url.getRelativeContentPath( obj )
            portal = obj.portal_url.getPortalObject()
            for i in range( len( relative ) ):
                now = relative[ :i+1 ]
                obj = portal.restrictedTraverse( now )
                if now[ -1 ] not in ignoreIds:
                    res.append( { 'id'      : now[ -1 ]
                                   , 'title'   : obj.title_or_id()
                                   , 'url'     : portal_url + '/' + '/'.join(now)
                                   }
                                )
        return res


    security.declarePublic( 'getStatus' )
    def getStatus(self, request=None):
        """ get boolean true (virtual host in use in current request) or false
        (nope) """
        if not request and hasattr(self, "REQUEST"):
            request=self.REQUEST
        return self.getVirtualHostFolder().getVirtualHostStatus(request)
        

def manage_dummyAddVirtualHostTool(*ignored, **alsoIgnored):
    raise NotImplementedError # INTENDED.  see Install script in Extensions folder.  The script is the only way to install this tool

InitializeClass(VirtualHostTool)
